#!/bin/bash
#
# Copyright (c) 2009, Intel Corporation.
#
# This Intel Cluster Ready program ("Software") is furnished under license and
# may only be used or copied in accordance with the terms of that license. No 
# license, express or implied, by estoppel or otherwise, to any intellectual 
# property rights is granted by this document. The Software is subject to 
# change without notice, and should not be construed as a commitment by Intel 
# Corporation to market, license, sell or support any product or technology. 
# Unless otherwise provided for in the license under which this Software is 
# provided, the Software is provided AS IS, with no warranties of any kind, 
# express or implied. Except as expressly permitted by the Software license, 
# neither Intel Corporation nor its suppliers assumes any responsibility or 
# liability for any errors or inaccuracies that may appear herein. Except as 
# expressly permitted by the Software license, no part of the Software may be 
# reproduced, stored in a retrieval system, transmitted in any form, or 
# distributed by any means without the express written consent of Intel
# Corporation.
#
#
# This script configures the IB0 interface
#

################# Set global vars used by the script ##################
#######################################################################
function set_default_params {
# Error handling codes
    E_XCD=66
    E_NOTROOT=67
    E_WRONGARGS=65
    E_FILENOTFOUND=4
    E_WRONGMD5=7
    E_BADSTATUS=17
    E_BADVER=27
    E_WRONGCHMOD=37
    E_CHKCFG=47
    E_SERVICE_FAIL=57
    E_BADNETWORK=77
    E_FAILMOD=87
    E_WRONG_DISTRO=97
    E_FAIL_LIC=107
    E_NO_IBDEV=108
    SUCCESS=0
# Script version
    VERSION='1.0'
# Global variables
    BASE_DIR='/opt/intel/icr/sbin' # have to be changed whit sourced of  conffile
    DIR_CFG='/etc'
    ROCKS_DIR='/export/rocks/install'
    NFS_DIR='/share/apps'
    ICR_HOME='/export/home/icr'
    ICR_RECIPE='S5520UR-ICR1.1-ROCKS+5.1-RH5.3-C1-v1.1'
    ICR_PCKG='S5520UR-ICR1.1-ROCKS+5.1-RH5.3-C1-v1.1-scripts.tar.gz'
# Default values
    IB0_IP=''
    IB0_NM=''
    IB0_SN=''
    CWD="$PWD"
# Date
    DATE=`date +_%k_%M_%S`
# Log File
    LOGFILE="/dev/null"
# Values by default
    NET_DIR="$DIR_CFG/sysconfig/network-scripts"
    CFG2="$NET_DIR/ifcfg-ib1"
    NET_NAME='ipoib'
    IFACE='ib0'
    FABRIC='igb'
    return 0
}


##################### Change dir in a correct way #####################
#######################################################################
function cd_sec {
    cd $1
    if [ "$PWD" != "$1" ]
    then
	echo "Can't change to $1"
	exit $E_XCD
    fi
    return 0
}

##################### Check net addres well formed ####################
#######################################################################
function check_address {
    ADD=$(echo $1 | grep '^[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}$')
    if [ "x$ADD" = "x" ]
    then
	echo "Network address $2 is not valid. Please specify a correct one"
	usage $0
	exit 1
    else
	a=$(echo $ADD | cut -d. -f1)
	b=$(echo $ADD | cut -d. -f2)
	c=$(echo $ADD | cut -d. -f3)
	d=$(echo $ADD | cut -d. -f4)
	for j in a b c d
	do
	    if [ ${!j} -lt 1 ] || [ ${!j} -gt 255 ]
	    then
		return 1
	    fi
	done
    fi
    return 0
}

################################ Help #################################
#######################################################################
function usage {
    echo "Usage: `basename $0` [OPTION] {[PARAMETERS]}"
    echo "Options:"
    echo "	-ip IPADDR,	--ipaddr IPADDR	IP	Address for infiniband network"
    echo "	-mask NETMASK,	--netmask NETMASK	Netmask address for infiniband network"
    echo "	-net NETNAME,	--netname NETNAME	Netname for infiniband network"
    echo "	-h,		--help			Display this help and exit"
    echo "	-v,		--version		Output version information and exit"
    echo ""
    return 0
}


################## Parameter Parser Function ##########################
#######################################################################
function parameters {
    while [ ! -z "$1" ]
    do
        case "$1" in
	    -h | --help)
		usage
		shift
		exit $SUCCESS
		;;
	    -ip | --ipaddr)
		IB0_IP=$2
		check_address $IB0_IP "ip"
		shift 2
		;;
	    -mask | --netmask)
		IB0_NM=$2
		check_address $IB0_NM "nm"
		shift 2
		;;
	    -net | --netname)
		NET_NAME=$2
		shift 2
		;;
	    -v | --version)
		echo "Version: $VERSION"
		exit $SUCCESS
		;;
	    *)
		usage
		shift
		exit $E_WRONGARGS
		;;
        esac
    done
    if [ ! -n "$IB0_IP" ] && [ ! -n "$IB0_NM" ]
    then
	DEFAULT=1
    else
	DEFAULT=0
    fi
}

################# CHECK IF ROOT IS RUNNING THE SCRIPT #################
#######################################################################
function root_check {
    ROOT_UID=0
    if [ "$UID" -ne "$ROOT_UID" ]
    then
	echo "Must be root to run this script"
	exit $E_NOTROOT
    fi
    return 0
}

#######################################################################
####################### Rocks+ IPoIB network ##########################
########################## IB configuration ###########################
#######################################################################

root_check
set_default_params
parameters $@

echo 'ipoib 1' >> $LOGFILE

echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=             Configuiring IB devices               ="
echo "====================================================="
echo ""
echo -ne '\033[37m'

cd_sec $BASE_DIR #point to scripts dir

# Infiniband device check
IB_DEV_COUNT=$(lspci | grep -i InfiniBand | wc -l)
if [ $IB_DEV_COUNT -eq "0" ] ; then
        echo "No supported InfiniBand adapter founded."
        exit $E_NO_IBDEV
fi

if [ $DEFAULT -eq 0 ]
then
    echo "Using entered data..."
    IB0_SN=$($BASE_DIR/icr_subnet_calc.sh $IB0_IP $IB0_NM)
    check_address $IB0_IP "ip"
    check_address $IB0_NM "nm"
    check_address $IB0_SN "sn"
    echo "Net name:   $NET_NAME"
    echo "Interface:  $IFACE"
    echo "IP address: $IB0_IP"
    echo "Netmask:    $IB0_NM"
    echo "Subnet:     $IB0_SN"
else
    echo "Using default values..."
    if ! (ifconfig eth0 | grep '[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}$' >>$LOGFILE 2>&1) ##lsmod | grep "$FABRIC" >>$LOGFILE 2>&1 && 
    then
	echo 'Could not get eth0 IP address information'
	exit $E_BADNETWORK
    else
	IB0_IP=$(/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}' | awk -F . '{print $1"."$2"."$3"."$4}')
	IB0_NM=$(/sbin/ifconfig eth0 | grep 'Mask:' | cut -d: -f4)
	IB0_IP=$($BASE_DIR/icr_next_ip.sh $IB0_IP $IB0_NM)
        IB0_SN=$($BASE_DIR/icr_subnet_calc.sh $IB0_IP $IB0_NM)
######### the following sanity checks should not be necessary ##########
	check_address $IB0_IP "ip"
	check_address $IB0_NM "nm"
	check_address $IB0_SN "sn"
	echo "Net name:   $NET_NAME"
	echo "Interface:  $IFACE"
	echo "IP address: $IB0_IP"
	echo "Netmask:    $IB0_NM"
	echo "Subnet:     $IB0_SN"
    fi
fi

##### source del conffile
##### sanity check de si la red esta siendo usada

# Setting up opensmd daemon  ## add sanity check
chkconfig opensmd on >>$LOGFILE 2>&1
if [ $? -ne 0 ]
then
    echo "A problem occur trying to configure opensmd"
    exit $E_CHKCFG
fi

# Network IPoIB add
rocks add network $NET_NAME subnet=$IB0_SN netmask=$IB0_NM >>$LOGFILE 2>&1

# Interface IPoIB load
rocks add host interface `hostname -s` $IFACE ip=$IB0_IP subnet=$NET_NAME name=`hostname -s`-ib >>$LOGFILE 2>&1

# IPoIB sync
echo "Syncing configuration on the frontend...(may take several minutes)"
rocks sync config >>$LOGFILE 2>&1
rocks sync host network $(hostname -s) >>$LOGFILE 2>&1
ifup $IFACE >>$LOGFILE 2>&1

# Configure ib1 if available
IB1_AVAILABLE=$(ifconfig -a | grep '^ib1')
if [ "x$IB1_AVAILABLE" = "xib1" ]; then
	echo -e "Configuring ib1 to be disabled on boot"
	echo "# Modify by Automata ICR Recipe" > $CFG2
	echo "DEVICE=ib1" >> $CFG2
	echo "BOOTPROTO=none" >> $CFG2
	echo "ONBOOT=no" >> $CFG2

fi
# Apply configuration
service network restart >>$LOGFILE 2>&1
if [ $? -ne 0 ]
then
    echo "There was a problem restarting network services"
    exit $E_BADNETWORK
fi

service opensmd restart >>$LOGFILE 2>&1
if [ $? -ne 0 ]
then
    echo "There was a problem restarting subnet manager service"
    exit $E_BADNETWORK
fi


# Validation
rocks list network | grep $NET_NAME >>$LOGFILE 2>&1
if [ $? -ne 0 ]
then
    echo "The $NET_NAME network could not be added to Rocks+ database"
    exit $E_BADNETWORK
fi

rocks list host interface `hostname -s` | grep $NET_NAME >>$LOGFILE 2>&1
if [ $? -ne 0 ]
then
    echo "The $IFACE interface could not be added to Rocks+ database"
    exit $E_BADNETWORK
fi

cd_sec $BASE_DIR


echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=               IB devices configured               ="
echo "====================================================="
echo ""
echo -ne '\033[37m'

echo 'ipoib 0' >> $LOGFILE
