#!/bin/bash
#
# Copyright (c) 2009, Intel Corporation.
#
# This Intel Cluster Ready program ("Software") is furnished under license and
# may only be used or copied in accordance with the terms of that license. No 
# license, express or implied, by estoppel or otherwise, to any intellectual 
# property rights is granted by this document. The Software is subject to 
# change without notice, and should not be construed as a commitment by Intel 
# Corporation to market, license, sell or support any product or technology. 
# Unless otherwise provided for in the license under which this Software is 
# provided, the Software is provided AS IS, with no warranties of any kind, 
# express or implied. Except as expressly permitted by the Software license, 
# neither Intel Corporation nor its suppliers assumes any responsibility or 
# liability for any errors or inaccuracies that may appear herein. Except as 
# expressly permitted by the Software license, no part of the Software may be 
# reproduced, stored in a retrieval system, transmitted in any form, or 
# distributed by any means without the express written consent of Intel
# Corporation.
#
# This script is intended to run Intel(R) Cluster Checker
#


# Provisioning vars
PROVISION='rocksplus'  ## change checking what prov is installed
PROV_VER='1.0'      ## once i have the prov find the version at know locations

# Script Version
VERSION='1.0'

# Error handling codes
E_FILENOTFOUND=4
E_WRONGARGS=65
E_BADSTATUS=17
SUCCESS=0

##################### Change dir in a correct way #####################
#######################################################################
function cd_sec {
    cd $1
    if [ "$PWD" != "$1" ]
    then
	echo "Can't change to $1"
	exit $E_XCD
    fi
    return 0
}


###################### Check a running process ########################
#######################################################################
function running_proc {
    if [ "x$1" == "x" ]
    then
        return 1
    else
        echo -n "Running........"
        echo -ne "\033[s" #save cursor position
        while ps axo pid | grep -e $1 >/dev/null
        do
            echo -n "|"
            sleep 0.1
            echo -ne "\033[u" #restore cursor position
            echo -n "/"
            sleep 0.1
            echo -ne "\033[u" #restore cursor position
            echo -n "-"
            sleep 0.1
            echo -ne "\033[u" #restore cursor position
            echo -n "\\"
            sleep 0.1
            echo -ne "\033[u" #restore cursor position
        done
        sleep 0.2
        echo -n "DONE"
        return 0
    fi
}

### CONF FILE ###
if [ -z $CONF_BOARD ]
then 
    echo "The Intel Cluster Ready Config file was not generated"
    echo "Trying again after execute the './"$PROVISION"_icr_install.sh --configure'"
    exit $E_FILENOTFOUND
fi
source $CONF_BOARD

echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=              Validating your cluster              ="
echo "====================================================="
echo ""
echo -ne '\033[37m'

# Create nodelist file
cd_sec $ICLCK_CONF_DIR
TEMP_NODELIST="/tmp/temp_nodelist.$$"
FRONTEND=$(hostname -s)
if [ "$PROVISION" == "rocksplus" ]
then
    if [ ! -f $NODELIST ] 
    then
	dbreport nodes > $NODELIST
	sed -i -e "s/\($FRONTEND.*\)/\1  # type:head/" $NODELIST
    else
	dbreport nodes > $TEMP_NODELIST
	sed -i -e "s/\($FRONTEND.*\)/\1  # type:head/" $TEMP_NODELIST
	echo -n "Checking nodelist file..."
	diff $NODELIST $TEMP_NODELIST >>$LOGFILE 2>&1
	if [ $? -ne 0 ]
	then
	    echo -ne '\033[31m'
	    echo "out of date!"
	    echo -ne '\033[37m'
	    echo -n "Making backup of nodelist file..."
	    cp $NODELIST "$NODELIST.orig" >>$LOGFILE 2>&1
	    if [ $? -eq 0 ]
	    then
		echo -ne '\033[32m'
		echo "OK"
		echo -ne '\033[37m'
	    else
		echo -ne '\033[31m'
		echo "FAIL"
		echo -ne '\033[37m'
		exit 75
	    fi
	    echo -n "Updating nodelist file..."
	    cp $TEMP_NODELIST $NODELIST >>$LOGFILE 2>&1
	    if [ $? -eq 0 ]
	    then
		echo -ne '\033[32m'
		echo "OK"
		echo -ne '\033[37m'
	    else
		echo -ne '\033[31m'
		echo "FAIL"
		echo -ne '\033[37m'
		exit 75
	    fi
	else
	    echo -ne '\033[32m'
	    echo "up to date!"
	    echo -ne '\033[37m'
	fi
    fi
elif [ "$PROVISION" == "pcm" ]
then
    echo "Nodelist creation for $PROVISION not implemented here yet"
    
else
    echo "The provisioning system used has not support for automatic nodelist creation"
fi
cd_sec $PACKAGES_FOLDER


## Run CLCK
if [ "$1" == "--verbose" ]
then
## Run deployment mode verification for user
    wait
    su - icr -c "$HOME_USER_FOLDER/$ICR_DEPLOYMENT"
## Run deployment mode verification for root
    wait
    sleep 2m # To allow date change and file not being replaced
    $PACKAGES_FOLDER/$ICR_DEPLOYMENT
else
## Run user-mode deployment verification
    echo ; echo -n "Running Intel Cluster Checker deployment mode for user: "
    echo -ne '\033[36m'
    echo "icr" ##harcoded user
    echo -ne '\033[37m'
    su - icr -c "$HOME_USER_FOLDER/$ICR_DEPLOYMENT" >>/dev/null 2>&1 & ##harcoded user
    CLCK_PID=$(jobs -p)
    running_proc $CLCK_PID
    wait $CLCK_PID
    CLCK_OUT=$(ls -rt $ICR_LOG_FOLDER | grep -e out | tail -n1)
	if [ -z $CLCK_OUT ]
	then
		echo ; echo "Unknow error. Aborting..."
		kill -9 $CLCK_PID &>/dev/null
		echo "Please run with --verbose switch"
		exit $E_BADSTATUS
	fi
    echo " (log file: $ICR_LOG_FOLDER/$CLCK_OUT)"
    echo -n "Check has "
    CLCK_RESULT=$(cat $ICR_LOG_FOLDER/$CLCK_OUT 2>/dev/null | grep -i -e 'check has' | grep -o -e FAILED)
    if [ $? -eq 0 ]
    then
	echo -ne '\033[31m'
	echo "$CLCK_RESULT"
	echo -ne '\033[37m'
    else
	CLCK_RESULT=$(cat $ICR_LOG_FOLDER/$CLCK_OUT 2>/dev/null | grep -i -e 'check has' | grep -o -e Succeeded)
        if [ $? -eq 0 ]
            then
            echo -ne '\033[32m'
            echo "$CLCK_RESULT"
            echo -ne '\033[37m'
        else
            CLCK_RESULT="FAILED"
            echo -ne '\033[31m'
            echo "$CLCK_RESULT"
            echo -ne '\033[37m'
	    echo "Please run with --verbose to see the errors."
        fi
    fi 
## Run root-mode deployment verification
    echo ; echo -n "Running Intel Cluster Checker deployment  mode for user: "
    echo -ne '\033[36m'
    echo "`whoami`" 
    echo -ne '\033[37m'
    sleep 2m # To allow date change and log file not being replaced
    $PACKAGES_FOLDER/$ICR_DEPLOYMENT >>/dev/null 2>&1 &
    CLCK_PID=$(jobs -p)
    running_proc $CLCK_PID
    wait $CLCK_PID
    CLCK_OUT=$(ls -rt $ICR_LOG_FOLDER | grep -e out | tail -n1)
	if [ -z $CLCK_OUT ]
	then
		echo ; echo "Unknow error. Aborting..."
		kill -9 $CLCK_PID &>/dev/null
		echo "Please run with --verbose switch"
		exit $E_BADSTATUS
	fi
	echo " (log file: $ICR_LOG_FOLDER/$CLCK_OUT)"
    echo -n "Check has "
    CLCK_RESULT=$(cat $ICR_LOG_FOLDER/$CLCK_OUT 2>/dev/null | grep -i -e 'check has' | grep -o -e FAILED)
    if [ $? -eq 0 ]
    then
	echo -ne '\033[31m'
	echo "$CLCK_RESULT"
	echo -ne '\033[37m'
    else
	CLCK_RESULT=$(cat $ICR_LOG_FOLDER/$CLCK_OUT 2>/dev/null | grep -i -e 'check has' | grep -o -e Succeeded)
	echo -ne '\033[32m'
	echo "$CLCK_RESULT"
	echo -ne '\033[37m'
    fi 
fi

echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=            Validation process finish              ="
echo "====================================================="
echo ""
echo -ne '\033[37m'
