#!/bin/bash
#
# Copyright (c) 2009, Intel Corporation.
#
# This Intel Cluster Ready program ("Software") is furnished under license and
# may only be used or copied in accordance with the terms of that license. No 
# license, express or implied, by estoppel or otherwise, to any intellectual 
# property rights is granted by this document. The Software is subject to 
# change without notice, and should not be construed as a commitment by Intel 
# Corporation to market, license, sell or support any product or technology. 
# Unless otherwise provided for in the license under which this Software is 
# provided, the Software is provided AS IS, with no warranties of any kind, 
# express or implied. Except as expressly permitted by the Software license, 
# neither Intel Corporation nor its suppliers assumes any responsibility or 
# liability for any errors or inaccuracies that may appear herein. Except as 
# expressly permitted by the Software license, no part of the Software may be 
# reproduced, stored in a retrieval system, transmitted in any form, or 
# distributed by any means without the express written consent of Intel
# Corporation.
#
#
# This script is intended to create all the configuration files required for Intel(R) Cluster Checker
#

# Provisioning vars
PROVISION='rocks+'  ## change checking what prov is installed
PROV_VER='1.0'      ## once i have the prov find the version at know locations

# Script Version
VERSION='1.0'

# Error handling codes
ROOT_UID=0
E_FILENOTFOUND=4
E_WRONGARGS=65
E_NOTROOT=67
SUCCESS=0
FAIL=0

################## Parameter Paser Function ###########################
#######################################################################
function parameters {
    while [ ! -z "$1" ]
    do
        case "$1" in
	    -h | --help)
		usage
		shift
		exit $SUCCESS
		;;
	    -r | --recipe)
		ICR_RUID=$2
		shift 2
		;;
	    -v | --version)
		echo "Version: $VERSION"
		exit $SUCCESS
		;;
	    *)
		usage
		shift
		exit $E_WRONGARGS
		;;
        esac
    done
}

############################ Script usage #############################
#######################################################################
function usage {
    echo "Usage: `basename $0` [OPTION] {[PARAMETERS]}"
    echo "Options:"
    echo "	-r RUID,	--recipe RUID		Install Intel(R) Cluster Ready Recipe ID"
    echo "	-h,		--help			Display this help and exit"
    echo "	-v,		--version		Output version information and exit"
    echo ""
    echo "Note: is strongly recomend that use the recipe installer and avoid to use this script standalone"
    echo ""
    return 0
}

########## Checks for the existance of a prerequisite file ############
#######################################################################
function set_global_parameters {
    if [ -z $CONF_BOARD ]
    then 
	echo "The Intel Cluster Ready Config file was not generated."
	echo "Trying to use defaults values..."
	if [ -z $ICR_RUID ]
	then
	    echo "Recipe UID not entered"
	    echo "Trying again after execute the './"$PROVISION"_install-v"$PROV_VER"'.sh --configure'"
	    echo
	    usage
	    exit $E_FILENOTFOUND
	fi
## Vars by default ##
# Error codes
	E_XCD=66
# Global vars
	PATH_LICENSES='/opt/intel/licenses'
	PACKAGES_FOLDER='/root'
	ICR_CONF_DIR='/etc/intel'
	CONF_FILE_POST=".conf"
	BOARD=$(dmidecode | grep -m 1 "Product Name" | awk '{ print $3 }')
	CONF_BOARD=$BOARD$CONF_FILE_POST
	CONF_FILE=$BOARD$CONF_FILE_POST
	HOME_USER_FOLDER='/home/icr'
	ICR_LOG_FOLDER='/var/log/icr'
	ICR_USER_NAME='icr'
	LOGFILE='/dev/null'
# CLCK vars
	ICLCK_CLCK_CONF_POST="-config.xml"
	ICLCK_HEAD_LIST_POST="-head.list"
	ICLCK_NODE_LIST_POST="-compute-node.list"
	ICLCK_CONF="$BOARD-$ICR_RUID$ICLCK_CLCK_CONF_POST"
	ICLCK_HEAD_LIST="$ICR_RUID$ICLCK_HEAD_LIST_POST"
	ICLCK_NODE_LIST="$ICR_RUID$ICLCK_NODE_LIST_POST"
	ICLCK_LIC='*.lic'
	ICLCK_CONF_DIR="$ICR_CONF_DIR/clck"
	NODELIST="nodelist"
#	USER_ICLCK_WELLNESS="$PACKAGES_FOLDER/icr_user_wellness.sh"
#	USER_ICLCK_COMPLIANCE="$PACKAGES_FOLDER/icr_user_compliance.sh"
	ICR_DEPLOYMENT="$PACKAGES_FOLDER/icr_deployment.sh"
    else
### CONF FILE ###
	source $CONF_BOARD
	##check if it has already been added the export vars into bashrc
	echo "export BOARD=$BOARD" >> $HOME_USER_FOLDER/.bashrc
	echo "export CONF_BOARD=$ICR_CONF_DIR/$CONF_FILE" >> $HOME_USER_FOLDER/.bashrc
	echo "export CONF_FILE=$ICR_CONF_DIR/$CONF_FILE" >> $HOME_USER_FOLDER/.bashrc
    fi
}

#####################    CHECK_FILE    ############################
###################################################################
#Checks for the existance of a prerequisite file
# $1 file $2 permissions(1=x 2=w 4=r)
# eg. check_file config.xml 777
# eg. check_file install.sh 755
function check_file {
    echo -n "Checking $1 ..."
    if [ ! -f $1 ]
    then
	echo -ne '\033[31m'
	echo "FAIL"
	echo -ne '\033[37m'
	FAIL=1
    else
	echo -ne '\033[32m'
	echo "OK"
	echo -ne '\033[37m'
	if [ ! -z $2 ]
	then
	    echo -n "Setting permissions for $1 ..."
	    chmod $2 $1 >>$LOGFILE 2>&1
	    if [ $? -ne 0 ]
	    then
		echo -ne '\033[31m'
		echo "FAIL"
		echo -ne '\033[37m'
		echo "Warning: `basename $0` could not set the permissions for $1"
		exit $E_FAILMOD
	    else
		echo -ne '\033[32m'
		echo "OK"
		echo -ne '\033[37m'
	    fi
	fi
    fi
}

##################### Check the check_file output #####################
#######################################################################
function check_file_status {
    if [ $FAIL -eq 1 ]
    then
	echo
	echo "WARNING: The required files are not present."
	echo "Please ensure that the reference implementation package files are available at $PACKAGES_FOLDER folder".
	echo
	FAIL=0
	exit $E_FILENOTFOUND
    fi
}

##################### Change dir in a correct way #####################
#######################################################################
function cd_sec {
    cd $1
    if [ "$PWD" != "$1" ]
    then
	echo "Can't change to $1"
	exit $E_XCD
    fi
    return 0
}

##################### Check that is a superuser #######################
#######################################################################
function root_check {
    if [ "$UID" -ne "$ROOT_UID" ]
    then
	echo "Must be root to run this script"
	exit $E_NOTROOT
    fi
    return 0
}

#######################################################################
#######################################################################
#######################################################################

root_check
parameters $@
set_global_parameters


echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=               Configuring ICR user                ="
echo "====================================================="
echo ""
echo -ne '\033[37m'


cd_sec $PACKAGES_FOLDER

FAIL=0
check_file $ICLCK_CONF
check_file $ICLCK_HEAD_LIST
check_file $ICLCK_NODE_LIST
check_file $ICLCK_LIC
check_file_status

if [ ! -d $ICLCK_CONF_DIR ]
then
    mkdir -p $ICLCK_CONF_DIR
fi

cd_sec $PACKAGES_FOLDER
cp -f $ICLCK_LIC $PATH_LICENSES >>$LOGFILE 2>&1
chmod o+r $PATH_LICENSES/*.lic >>$LOGFILE 2>&1

# Creates de folder for storing Intel(R) Cluster Checker's results
if [ ! -d $ICR_LOG_FOLDER ] ## log folders
then
    mkdir -p $ICR_LOG_FOLDER
    chown $ICR_USER_NAME.$ICR_USER_NAME $ICR_LOG_FOLDER
else
    echo "$ICR_LOG_FOLDER folder already created, skipping step"
fi

## generar las keys desde root y copiarlas a manopla a user
cat >> $HOME_USER_FOLDER/.bashrc <<EOF
# User specific aliases and functions
if [ ! -f ~/.ssh/id_rsa ]; then
    echo 'No public/private RSA keypair found.'
    ssh-keygen -t rsa -b 2048 -f ~/.ssh/id_rsa -N ""
    cat ~/.ssh/id_rsa.pub > ~/.ssh/authorized_keys
    chmod 644 ~/.ssh/authorized_keys
fi
EOF

rocks sync config >>$LOGFILE 2>&1

cd_sec $PACKAGES_FOLDER

cp -f $ICLCK_CONF $ICLCK_HEAD_LIST $ICLCK_NODE_LIST $ICLCK_CONF_DIR >>$LOGFILE 2>&1
if [ -f $ICLCK_CONF_DIR/config.xml ]
then
    rm -f $ICLCK_CONF_DIR/config.xml
fi

ln -s $ICLCK_CONF_DIR/$ICLCK_CONF $ICLCK_CONF_DIR/config.xml >>$LOGFILE 2>&1

# Copy Automata CLCK script (compliance and wellnes)
cp -f $ICR_DEPLOYMENT $HOME_USER_FOLDER >>$LOGFILE 2>&1
chown $ICR_USER_NAME.$ICR_USER_NAME $HOME_USER_FOLDER/$ICR_DEPLOYMENT >>$LOGFILE 2>&1
FAIL=0
check_file $HOME_USER_FOLDER/$ICR_DEPLOYMENT 555
check_file_status

echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=                ICR user Configured                ="
echo "====================================================="
echo ""
echo -ne '\033[37m'
