#!/bin/bash
#
# Copyright (c) 2009, Intel Corporation.
#
# This Intel Cluster Ready program ("Software") is furnished under license and
# may only be used or copied in accordance with the terms of that license. No 
# license, express or implied, by estoppel or otherwise, to any intellectual 
# property rights is granted by this document. The Software is subject to 
# change without notice, and should not be construed as a commitment by Intel 
# Corporation to market, license, sell or support any product or technology. 
# Unless otherwise provided for in the license under which this Software is 
# provided, the Software is provided AS IS, with no warranties of any kind, 
# express or implied. Except as expressly permitted by the Software license, 
# neither Intel Corporation nor its suppliers assumes any responsibility or 
# liability for any errors or inaccuracies that may appear herein. Except as 
# expressly permitted by the Software license, no part of the Software may be 
# reproduced, stored in a retrieval system, transmitted in any form, or 
# distributed by any means without the express written consent of Intel
# Corporation.
#
#
# This script is intended to reduce the gap between the actual cluster configuration
# and Intel Cluster Ready specification
#

# Error handling codes
E_XCD=66
E_NOTROOT=67
E_WRONGARGS=65
E_FILENOTFOUND=4
E_WRONGMD5=7
E_BADSTATUS=17
E_BADVER=27
E_WRONGCHMOD=37
E_CHKCFG=47
E_SERVICE_FAIL=57
E_BADNETWORK=77
E_FAILMOD=87
E_WRONG_DISTRO=97
E_FAIL_LIC=107
E_NO_IBDEV=108
SUCCESS=0
RETVAL=0
FAIL=0

# Define colors
red='\033[31m'
green='\033[32m'
yellow='\033[33m'
blue='\033[34m'
pink='\033[35m'
cian='\033[36m'
white='\033[37m'

################################################### 
VERSION=1.1  # Script version

PROV="ROCKS+"
PROV_VER="5.1"
OS_VER="RH5.3"
ROOT_UID=0
E_BADNETWORK=77
###################################################
PACKAGES_FOLDER="/opt/intel/icr/sbin"
LICENSE_FOLDER="/root"
ICR_CONF_DIR="/etc/intel"
SHARED_FOLDER="/shared/apps"
HOME_FOLDER="/home"
ICR_USER_NAME="icr"
HOME_USER_FOLDER=$HOME_FOLDER/$ICR_USER_NAME
ICR_LOG_FOLDER="/var/log/icr"
LOGFILE="/dev/null" 
ICLCK_RESULTS_FOLDER="/var/log/icr"
###################################################
##############  ICLCK Filenames ###################
CONF_FILE_POST=".conf"
ICLCK_CLCK_CONF_POST="-config.xml"
ICLCK_HEAD_LIST_POST="-head.list"
ICLCK_NODE_LIST_POST="-compute-node.list"
ICR_SCRIPTS_POST="-scripts.tar.gz"
ICLCK_LIC="*.lic"
ICLCK_VER="1.3"
ICLCK_BIN_DIR="/opt/intel/clck/$ICLCK_VER"
ICLCK_CONF_DIR="$ICR_CONF_DIR/clck"
NODELIST="nodelist"
PATH_LICENSES='/opt/intel/licenses'
###################################################
##########  LINUX OS config Filenames   ###########
CFG="/etc/sysconfig/network-scripts/ifcfg-ib0"
CFG2="/etc/sysconfig/network-scripts/ifcfg-ib1"

###################################################
############# ICR script section start ############
ICR_IB_CONF="icr_ib_conf.sh"
ICR_USER_CONF="icr_user_conf.sh"
ICR_CLCK_EXEC="icr_passthrough.sh"
ICR_DEPLOYMENT="icr_deployment.sh"
ICR_PROV_CHECK="icr_check_rocks_components.sh" 
SUBNET_CALC="icr_subnet_calc.sh"
ICR_NEXT_IP="icr_next_ip.sh"
############# ICR script section finish ###########
###################################################

########## Check net addres well formed ###########
###################################################
function check_address {
    ADD=$(echo $1 | grep '^[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}$')
    if [ "x$ADD" = "x" ]
    then
	return 1
    else
	a=$(echo $ADD | cut -d. -f1)
	b=$(echo $ADD | cut -d. -f2)
	c=$(echo $ADD | cut -d. -f3)
	d=$(echo $ADD | cut -d. -f4)
	for j in a b c d
	do
	    if [ ${!j} -lt 0 ] || [ ${!j} -gt 255 ]
	    then
		return 1
	    fi
	done
    fi
    return 0
}

########## Change dir in a correct way ############
###################################################
function cd_sec {
    cd $1
    if [ "$PWD" != "$1" ]
    then
	echo "Can't change to $1"
	exit $E_XCD
    fi
    return 0
}

########### Parameter Parser Function #############
###################################################
function parameters {
    if [ -z "$1" ]
    then
	usage
	exit $SUCCESS
    fi
    while [ ! -z "$1" ]
    do
	case "$1" in
	    -h | --help)
		usage
		shift
		exit $SUCCESS
		;;
	    -ip | --ipaddr)
		IB0_IP=$2
		check_address $IB0_IP
		if [ $? -eq 1 ]
		then
		    echo "`basename $0`: Bad network address $IB0_IP"
		    echo "Try '"`basename $0`" --help' for more information."
		    exit $E_BADNETWORK
		fi
		shift 2
		;;
	    -nm | --netmask)
		IB0_NM=$2
		check_address $IB0_NM
		if [ $? -eq 1 ]
		then
		    echo "`basename $0`: Bad network address $IB0_NM"
		    echo "Try '"`basename $0`" --help' for more information."
		    exit $E_BADNETWORK
		fi
		shift 2
		;;
	    -r | --recipe)
		ICLCK_RUID=$2
		if [ "$ICLCK_RUID" == "" ]
		then
		    echo "`basename $0`: missing RUID"
		    echo "Try '"`basename $0`" --help' for more information."
		    exit $E_WRONGARGS
		fi
		shift 2
		;;
	    -p | --path)
		LICENSE_FOLDER=$2
		shift 2
		;;
	    -c | --configure)
		if [ "$ICLCK_RUID" == "" ]
		then
		    echo "`basename $0`: missing RUID"
		    echo "Try '"`basename $0`" --help' for more information."
		    exit $E_WRONGARGS
		fi
		echo "Warning! No parameters were detected. Defaults values will be applied"
		check_and_configure
		exit $RET_VAL
		;;
	    -v | --version)
		echo "Version: $VERSION"
		exit $SUCCESS
		;;
	    *)
		echo "`basename $0`: Unrecognized option $1"
		echo "Try '"`basename $0`" --help' for more information."
		shift
		exit $E_WRONGARGS
		;;
	esac
    done
    cd_sec $PACKAGES_FOLDER
    install_recipe
    RETVAL=$?
}

##### CHECK IF ROOT IS RUNNING THE SCRIPT #########
###################################################
function root_check {
    if [ "$UID" -ne "$ROOT_UID" ]
    then
	echo "Must be root to run this script."
	exit $E_NOTROOT
    fi
    return 0
}

################    CHECK_FILE    #################
###################################################
#Checks for the existance of a prerequisite file
# $1 file $2 permissions(1=x 2=w 4=r)
# eg. check_file config.xml 777
# eg. check_file install.sh 755
function check_file {
    echo -n "Checking $1 ..."
    if [ ! -f $1 ]
    then
	echo -ne '\033[31m'
	echo "FAIL"
	echo -ne '\033[37m'
	FAIL=1
    else
	echo -ne '\033[32m'
	echo "OK"
	echo -ne '\033[37m'
	if [ ! -z $2 ]
	then
	    echo -n "Setting permissions for $1 ..."
	    chmod $2 $1 >>$LOGFILE 2>&1
	    if [ $? -ne 0 ]
	    then
		echo -ne '\033[31m'
		echo "FAIL"
		echo -ne '\033[37m'
		echo "Warning: `basename $0` could not set the permissions for $1"
		exit $E_FAILMOD
	    else
		echo -ne '\033[32m'
		echo "OK"
		echo -ne '\033[37m'
	    fi
	fi
    fi
}

####### Check status of check_file function #######
###################################################
function check_file_status {
    if [ $FAIL -eq 1 ]
    then
	echo
	echo "WARNING: The required files are not present."
	echo "Please ensure that the reference implementation package files are available at $PACKAGES_FOLDER folder".
	echo
	FAIL=0
	exit $E_FILENOTFOUND
    fi
}

###################### Help #######################
###################################################
function usage {
    echo "Usage: `basename $0` -r RUID [OPTION] {[PARAMETERS]}"
    echo "Options:"
    echo "      -r RUID,        --recipe RUID           Install Intel(R) Cluster Ready Reference Recipe ID." 
    echo "      -p PATH,        --path_packages PATH    The path where the scrips and packages Intel(R) Cluster Ready Reference Recipe are located."
    echo "      -c,             --configure             Check Intel(R) Cluster Ready Recipe files and generate the configuration file."
    echo ""
    echo "Parameters:"
    echo "      -ip IPADDR,      --ipaddr IPADDR        IP Address for infiniband network."
    echo "      -mask NETMASK,   --netmask NETMASK      Netmask address for infiniband network."
    echo "      -h,              --help                 Display this help and exit."
    echo "      -v,              --version              Output version information and exit."
    echo ""
    return 0
}


################    CHECK_FILE    #################
###################################################
#Checks for the existance of a prerequisite file
function check_file_scripts {
    FAIL=0
    check_file $ICR_IB_CONF 555
    check_file $ICR_USER_CONF 555
    check_file $ICR_CLCK_EXEC 555
    check_file $ICR_DEPLOYMENT 555
    check_file $LICENSE_FOLDER/$ICLCK_LIC 444
    check_file $SUBNET_CALC 555
    check_file $ICR_PROV_CHECK 555
    check_file $ICR_NEXT_IP 555
}


###############   Check PRE_PROV   ################
###################################################
function check_node_installer {

    echo -ne '\033[32m'
    echo ""
    echo "====================================================="
    echo "=    Checking the installation of essential rolls   ="
    echo "====================================================="
    echo ""
    echo -ne '\033[37m'

    ./$ICR_PROV_CHECK
    if [ $? -ne 0 ]
    then
	exit 1
    fi
    wait
}


###############   Check PRE_PROV   ################
###################################################
function check_and_configure {

    echo -ne '\033[32m'
    echo ""
    echo "====================================================="
    echo "=               Checking ICR files                  ="
    echo "====================================================="
    echo ""
    echo -ne '\033[37m'


############# 	CLCK Variables Generated ##############
    ICLCK_RESULTS_FOLDER=$ICLCK_RESULTS_FOLDER 
    ICLCK_CONF="$BOARD-$ICLCK_RUID$ICLCK_CLCK_CONF_POST"
    ICLCK_HEAD_LIST=$ICLCK_RUID$ICLCK_HEAD_LIST_POST
    ICLCK_NODE_LIST=$ICLCK_RUID$ICLCK_NODE_LIST_POST
    ICR_SCRIPTS=$ICLCK_RUID$ICR_SCRIPTS_POST

    FAIL=0
    check_file $ICLCK_CONF 444
    check_file $ICLCK_HEAD_LIST 444
    check_file $ICLCK_NODE_LIST 444
    check_file_scripts  
    check_file_status
    
    check_node_installer

    echo -ne '\033[32m'
    echo ""
    echo "====================================================="
    echo "=       Generating ICR Configuration Board File     ="
    echo "====================================================="
    echo ""
    echo -ne '\033[37m'

    if [ ! -d $ICR_CONF_DIR ]
    then
	mkdir -p $ICR_CONF_DIR
    fi

    cat > $ICR_CONF_DIR/$CONF_FILE <<EOC
###########################################################
########## GLOBAL RECIPE CONFIGURATION PARAMETER ##########
###########################################################

EOC

    echo "#################	ICR GLOBAL ESPECIFIC #####################" >> $ICR_CONF_DIR/$CONF_FILE 
    echo "ICR_CONF_DIR=$ICR_CONF_DIR" >> $ICR_CONF_DIR/$CONF_FILE
    echo "CONF_FILE=$CONF_FILE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "export CONF_BOARD=$ICR_CONF_DIR/$CONF_FILE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "BOARD=$BOARD" >> $ICR_CONF_DIR/$CONF_FILE
    echo "PROV=$PROV" >> $ICR_CONF_DIR/$CONF_FILE
    echo "PROV_VER=$PROV_VER" >> $ICR_CONF_DIR/$CONF_FILE
    echo "OS_VER=$OS_VER">> $ICR_CONF_DIR/$CONF_FILE
    echo "ICR_USER_NAME=$ICR_USER_NAME" >> $ICR_CONF_DIR/$CONF_FILE

    echo "#################	ICR Compliance ESPECIFIC #####################" >> $ICR_CONF_DIR/$CONF_FILE 
    echo "PACKAGES_FOLDER=$PACKAGES_FOLDER" >> $ICR_CONF_DIR/$CONF_FILE
    echo "SHARED_FOLDER=$SHARED_FOLDER" >> $ICR_CONF_DIR/$CONF_FILE	
    echo "HOME_USER_FOLDER=/home/$ICR_USER_NAME" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICR_DEPLOYMENT=$ICR_DEPLOYMENT" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ROOT_ICLCK_WELLNESS=$ROOT_ICLCK_WELLNESS" >> $ICR_CONF_DIR/$CONF_FILE
    echo "USER_ICLCK_COMPLIANCE=$USER_ICLCK_COMPLIANCE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "USER_ICLCK_WELLNESS=$USER_ICLCK_WELLNESS" >> $ICR_CONF_DIR/$CONF_FILE

    echo "SYSCTL_SCRIPT=$SYSCTL_SCRIPT" >> $ICR_CONF_DIR/$CONF_FILE
    echo "OFED_CONF_SCRIPT=$OFED_CONF_SCRIPT" >>  $ICR_CONF_DIR/$CONF_FILE
    echo "LIMITS_SCRIPT=$LIMITS_SCRIPT" >> $ICR_CONF_DIR/$CONF_FILE
    echo "JAVA_SCRIPT=$JAVA_SCRIPT" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ETH_SCRIPT=$ETH_SCRIPT" >> $ICR_CONF_DIR/$CONF_FILE
    echo "SYSCTL_FILE=$SYSCTL_FILE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "LIMITS_FILE=$LIMITS_FILE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "SUBNET_CALC=$SUBNET_CALC"  >> $ICR_CONF_DIR/$CONF_FILE
    echo '' >> $ICR_CONF_DIR/$CONF_FILE

    echo "#################	ICR Compliance ESPECIFIC #####################" >> $ICR_CONF_DIR/$CONF_FILE 
    echo "ICLCK_BIN_DIR=$ICLCK_BIN_DIR" >> $ICR_CONF_DIR/$CONF_FILE
    echo "LOGFILE=$LOGFILE" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICR_LOG_FOLDER=$ICR_LOG_FOLDER" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_CONF_DIR=$ICLCK_CONF_DIR" >> $ICR_CONF_DIR/$CONF_FILE
    echo "NODELIST=$NODELIST" >> $ICR_CONF_DIR/$CONF_FILE
    echo "PATH_LICENSES=$PATH_LICENSES" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_LIC=$LICENSE_FOLDER/$ICLCK_LIC" >> $ICR_CONF_DIR/$CONF_FILE 
    echo '' >> $ICR_CONF_DIR/$CONF_FILE 
    echo "PROVISIONING=$PROV$PROV_VER" >> $ICR_CONF_DIR/$CONF_FILE
    echo '' >> $ICR_CONF_DIR/$CONF_FILE
    echo "HOME_FOLDER=/home" >> $ICR_CONF_DIR/$CONF_FILE
    echo '' >> $ICR_CONF_DIR/$CONF_FILE


    echo "################# 	CLCK Variables Generated ##############" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_RESULTS_FOLDER=$ICLCK_RESULTS_FOLDER" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_CONF=$ICLCK_CONF" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_HEAD_LIST=$ICLCK_HEAD_LIST" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICLCK_NODE_LIST=$ICLCK_NODE_LIST" >> $ICR_CONF_DIR/$CONF_FILE
    echo "ICR_SCRIPTS=$ICR_SCRIPTS" >> $ICR_CONF_DIR/$CONF_FILE
    echo '' >> $ICR_CONF_DIR/$CONF_FILE

    echo "################# IB Configuration Variables #####################" >> $ICR_CONF_DIR/$CONF_FILE 
    echo "CFG=$CFG" >> $ICR_CONF_DIR/$CONF_FILE
    echo "CFG2=$CFG2" >> $ICR_CONF_DIR/$CONF_FILE
    echo "IBCONF=$IBCONF" >> $ICR_CONF_DIR/$CONF_FILE
    echo "DAT_CONF=$DAT_CONF" >> $ICR_CONF_DIR/$CONF_FILE
    echo "MODPROBEFILE=$MODPROBEFILE" >> $ICR_CONF_DIR/$CONF_FILE

    echo "IB0_IP=$IB0_IP" >> $ICR_CONF_DIR/$CONF_FILE
    echo "IB0_NM=$IB0_NM" >> $ICR_CONF_DIR/$CONF_FILE
    echo "export BOARD=$BOARD" >> ~/.bashrc
    echo "export CONF_BOARD=$ICR_CONF_DIR/$CONF_FILE" >> ~/.bashrc

    source $ICR_CONF_DIR/$CONF_FILE
    export compute_icr_group
    export BOARD
    export PACKAGES_FOLDER
    export CONF_BOARD=$ICR_CONF_DIR/$CONF_FILE
}


#################    RUNNING    ###################
###################################################
function automate_scripts {
    chmod a+x $ICR_IB_CONF
    echo -n "Running: "
    if [ "x$IB0_IP" = "x" ] && [ "x$IB0_NM" = "x" ]
    then
	echo -ne '\033[36m'
	echo "./$ICR_IB_CONF"
	echo -ne '\033[37m'
	./$ICR_IB_CONF 
    else
	echo -ne '\033[36m'
	echo "./$ICR_IB_CONF -ip $IB0_IP -mask $IB0_NM -net ipoib"
	echo -ne '\033[37m'
	./$ICR_IB_CONF -ip $IB0_IP -mask $IB0_NM -net "ipoib"
    fi
    wait
    chmod a+x $ICR_USER_CONF
    echo -n "Running: "
    echo -ne '\033[36m'
    echo "./$ICR_USER_CONF"
    echo -ne '\033[37m'
    ./$ICR_USER_CONF
    wait
}


#############   Install the recipe   ##############
###################################################
function install_recipe {
    check_and_configure
    wait
    automate_scripts
}

root_check

####### Retrieve Platform Name from the FRU #######
###################################################
BOARD=$(dmidecode | grep -m 1 "Product Name" | awk '{ print $3 }')
CONF_FILE=$BOARD$CONF_FILE_POST
###################################################

parameters $@

echo -ne '\033[32m'
echo ""
echo "====================================================="
echo "=          `basename $0`: finished       ="
echo "====================================================="
echo ""
echo -ne '\033[37m'

exit $RETVAL
